<?php

namespace Drupal\kinescope\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Defines the 'kinescope item' field type.
 *
 * @FieldType(
 *   id = "kinescope",
 *   label = @Translation("Kinescope"),
 *   category = @Translation("General"),
 *   default_widget = "kinescope_widget",
 *   default_formatter = "kinescope_default"
 * )
 */
class KinescopeItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    return empty($this->video_url);
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['video_url'] = DataDefinition::create('string')->setLabel(t('Video URL'));
    $properties['video_id'] = DataDefinition::create('string')->setLabel(t('Video ID'));
    $properties['is_responsive'] = DataDefinition::create('boolean')->setLabel(t('Is responsive'));
    $properties['width'] = DataDefinition::create('integer')->setLabel(t('Width'));
    $properties['height'] = DataDefinition::create('integer')->setLabel(t('Height'));
    $properties['options'] = DataDefinition::create('string')->setLabel(t('Options'));

    return $properties;
  }

  /**
   * {@inheritDoc}
   */
  public static function mainPropertyName() {
    return 'video_url';
  }

  /**
   * {@inheritdoc}
   */
  public function getConstraints() {
    $constraints = parent::getConstraints();
    $options['video_url']['NotBlank'] = [];
    $constraint_manager = \Drupal::typedDataManager()->getValidationConstraintManager();
    $constraints[] = $constraint_manager->create('ComplexData', $options);
    return $constraints;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $columns = [
      'video_url' => [
        'type' => 'varchar',
        'length' => 2083,
        'not null' => TRUE,
        'description' => 'The Kinescope video URL.',
        'default' => '',
      ],
      'video_id' => [
        'type' => 'varchar',
        'length' => 64,
        'not null' => TRUE,
        'description' => 'The Kinescope video ID.',
        'default' => '',
      ],
      'is_responsive' => [
        'type' => 'int',
        'size' => 'tiny',
        'description' => 'Whether the player is responsive.',
        'default' => 1,
      ],
      'width' => [
        'type' => 'int',
        'size' => 'small',
        'description' => 'The video width.',
        'unsigned' => TRUE,
      ],
      'height' => [
        'type' => 'int',
        'size' => 'small',
        'description' => 'The video height.',
        'unsigned' => TRUE,
      ],
      'options' => [
        'type' => 'varchar',
        'length' => 512,
        'description' => 'Options.',
      ],
    ];

    return [
      'columns' => $columns,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition) {
    $values = [
      'video_url' => 'https://kinescope.io/iNrkAyTsrA8KpzbZ83AeJi',
      'video_id' => '9EYv8kghsRHkiNqp6cj7ei',
      'is_responsive' => TRUE,
      'width' => 640,
      'height' => 480,
      'options' => 'autoplay; fullscreen; picture-in-picture; encrypted-media; gyroscope; accelerometer; clipboard-write; screen-wake-lock;',
    ];

    return $values;
  }

}
